<?php
// 关闭错误报告
error_reporting(0);

// 导入配置
$config = require_once(__DIR__ . '/config.php');

// 配置异常则退出
if (!is_array($config)) {
    exit('配置文件异常！');
}

// 将配置中的 DIR 数组定义为常量
foreach ($config['DIR'] as $k => $v) {
    define($k, $v);
}

// 初始化解密器
$Decryptor = new Decryptor($config['appid'], $config['token']);

if ($_GET['act'] == 'decrypt') { // 解密请求
    echo json_encode($Decryptor->decryptFile($_FILES['file']), 320); // 输出解密结果
} elseif ($_GET['act'] == 'download') { // 下载请求
    // 构建文件路径
    $file_path = ROOT . '/' . rtrim(DIR, '/') . '/' . rtrim(DECODE, '/') . '/' . md5($_GET['token'] . $config['token']);

    if (!file_exists($file_path) || (time() - filectime($file_path)) > TIMEOUT) { // 文件不存在或已过期
        exit('文件不存在或已过期！');
    }

    // 下载文件
    header('Content-Type: application/octet-stream');
    header('Content-Disposition: attachment; filename="' . basename($file_path) . '.php"');
    header('Content-Length: ' . filesize($file_path));
    readfile($file_path);
} else { // 非法调用
    exit('非法调用！');
}

class Decryptor {
    private $appId;
    private $appKey;

    public function __construct($appId, $appKey) {
        $this->appId = $appId;
        $this->appKey = $appKey;
        $this->token=md5(time().mt_rand());
        $this->file_name=md5($this->token.$this->appKey);
        $this->encodePath =ROOT. '/' . rtrim(DIR, '/') . '/' . rtrim(ENCODE, '/'). '/' . $this->file_name;
        $this->decodePath =ROOT. '/' . rtrim(DIR, '/') . '/' . rtrim(DECODE, '/') . '/' . $this->file_name;
   }

    public function decryptFile($file) {
        if (!$this->isValidFile($file)) {
            return ['code' => 201,'msg' => '文件不合法或大小超出限制！'];
        }
        if (move_uploaded_file($file['tmp_name'], $this->encodePath)) {
            return $this->decode();
        } else {
            return ['code' => 205, 'msg' => '文件移动失败！'];
        }
    }

    private function isValidFile($file) {
        return $file['size'] > 0 && $file['size'] <= 1024 * 1024;
    }

    private function decode() {
        $url = 'https://cloud.xiaojieapi.com/ajax/decrypt';
        $paras = [
            'post' => [
                // 用户id
                'appid' => $this->appId,
                // 接口密钥
                'token' => $this->appKey,
                // 加密类型，可选，留空则自动判断
                'type' => '',
                // 文件
                'file' => new CURLFile(realpath($this->encodePath)),
            ],
        ];
        $data = $this->curl($url, $paras);
        $arr = json_decode($data, true);
        if (empty($arr)) {
            return ['code' => 203, 'msg' => '解密服务器异常！'];
        } else if ($arr['code'] == 200) {
            file_put_contents($this->decodePath, $this->curl($arr['result']['url']));
            return ['code' => 200, 'msg' => '解密成功！', 'url' => './ajax.php?act=download&token='.$this->token];
        } else {
            return ['code' => 204,'msg' => $arr['error']];
        }
    }

    private function curl($url, $paras = []) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        if (isset($paras['Header'])) {
                $Header = $paras['Header'];
        } else {
                $Header[] = "Accept:*/*";
                $Header[] = "Accept-Encoding:gzip,deflate,sdch";
                $Header[] = "Accept-Language:zh-CN,zh;q=0.8";
                $Header[] = "Connection:close";
        }
        curl_setopt($ch, CURLOPT_HTTPHEADER, $Header);
        if (isset($paras['post'])) {
                curl_setopt($ch, CURLOPT_POST, 1);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $paras['post']);
        }
        if (isset($paras['header'])) {
                curl_setopt($ch, CURLOPT_HEADER, true);
        }
        curl_setopt($ch, CURLOPT_ENCODING, "gzip");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $ret = curl_exec($ch);
        curl_close($ch);
        return $ret;
    }
}
